@@ -133,10 +133,6 @@
         <source>&amp;Overview</source>
         <translation>&amp;Vista previa</translation>
     </message>
-    <message>
-        <source>Node</source>
-        <translation>Nodo</translation>
-    </message>
     <message>
         <source>Show general overview of wallet</source>
         <translation>Mostrar la vista previa general de la cartera</translation>
@@ -181,14 +177,6 @@
         <source>&amp;Change Passphrase...</source>
         <translation>&amp;Cambiar contraseña...</translation>
     </message>
-    <message>
-        <source>&amp;Sending addresses...</source>
-        <translation>Direcciones de &amp;envío...</translation>
-    </message>
-    <message>
-        <source>&amp;Receiving addresses...</source>
-        <translation>Direcciones de &amp;recepción...</translation>
-    </message>
     <message>
         <source>Open &amp;URI...</source>
         <translation>Abrir &amp;URL...</translation>
@@ -225,10 +213,6 @@
         <source>Bitcoin</source>
         <translation>Bitcoin</translation>
     </message>
-    <message>
-        <source>Wallet</source>
-        <translation>Cartera</translation>
-    </message>
     <message>
         <source>&amp;Send</source>
         <translation>&amp;Enviar</translation>
@@ -401,10 +385,6 @@
         <source>Options</source>
         <translation>Opciones</translation>
     </message>
-    <message>
-        <source>Active command-line options that override above options:</source>
-        <translation>Activar las opciones de linea de comando que sobre escriben las siguientes opciones:</translation>
-    </message>
     <message>
         <source>W&amp;allet</source>
         <translation>Cartera</translation>
@@ -636,6 +616,9 @@
 <context>
     <name>UnitDisplayStatusBarControl</name>
     </context>
+<context>
+    <name>WalletController</name>
+    </context>
 <context>
     <name>WalletFrame</name>
     </context>
