@@ -121,10 +121,6 @@
         <source>&amp;Change Passphrase...</source>
         <translation>&amp;Alilan ing Passphrase...</translation>
     </message>
-    <message>
-        <source>&amp;Receiving addresses...</source>
-        <translation>Address king pamag-Tanggap</translation>
-    </message>
     <message>
         <source>Send coins to a Bitcoin address</source>
         <translation>Magpadalang barya king Bitcoin address</translation>
@@ -153,10 +149,6 @@
         <source>Bitcoin</source>
         <translation>Bitcoin</translation>
     </message>
-    <message>
-        <source>Wallet</source>
-        <translation>Wallet</translation>
-    </message>
     <message>
         <source>&amp;Show / Hide</source>
         <translation>&amp;Ipalto / Isalikut</translation>
@@ -209,6 +201,10 @@
         <source>Up to date</source>
         <translation>Makatuki ya king aldo</translation>
     </message>
+    <message>
+        <source>&amp;Window</source>
+        <translation>&amp;Awang</translation>
+    </message>
     <message>
         <source>Catching up...</source>
         <translation>Catching up...</translation>
@@ -724,6 +720,9 @@
 <context>
     <name>UnitDisplayStatusBarControl</name>
     </context>
+<context>
+    <name>WalletController</name>
+    </context>
 <context>
     <name>WalletFrame</name>
     </context>
