@@ -238,10 +238,6 @@ Ví của bạn chưa được mã hóa.</translation>
         <source>&amp;Overview</source>
         <translation>&amp;Tổng quan</translation>
     </message>
-    <message>
-        <source>Node</source>
-        <translation>Node</translation>
-    </message>
     <message>
         <source>Show general overview of wallet</source>
         <translation>Hiện thỉ thông tin sơ lược chung về Ví</translation>
@@ -298,14 +294,6 @@ Ví của bạn chưa được mã hóa.</translation>
         <source>&amp;Change Passphrase...</source>
         <translation>&amp;Thay đổi mật khẩu...</translation>
     </message>
-    <message>
-        <source>&amp;Sending addresses...</source>
-        <translation>&amp;Địa chỉ gửi</translation>
-    </message>
-    <message>
-        <source>&amp;Receiving addresses...</source>
-        <translation>Địa chỉ nhận</translation>
-    </message>
     <message>
         <source>Open &amp;URI...</source>
         <translation>Mở &amp;URI...</translation>
@@ -358,10 +346,6 @@ Ví của bạn chưa được mã hóa.</translation>
         <source>Bitcoin</source>
         <translation>Bitcoin</translation>
     </message>
-    <message>
-        <source>Wallet</source>
-        <translation>Ví</translation>
-    </message>
     <message>
         <source>&amp;Send</source>
         <translation>&amp;Gửi</translation>
@@ -676,10 +660,6 @@ Ví của bạn chưa được mã hóa.</translation>
         <source>&amp;Main</source>
         <translation>&amp;Chính</translation>
     </message>
-    <message>
-        <source>MB</source>
-        <translation>MB</translation>
-    </message>
     <message>
         <source>IP address of the proxy (e.g. IPv4: 127.0.0.1 / IPv6: ::1)</source>
         <translation>Địa chỉ IP của proxy (ví dụ IPv4: 127.0.0.1 / IPv6: ::1)</translation>
@@ -1057,10 +1037,6 @@ Ví của bạn chưa được mã hóa.</translation>
         <source>Hide</source>
         <translation>Ẩn</translation>
     </message>
-    <message>
-        <source>(read the tooltip)</source>
-        <translation>(Đọc hướng dẫn)</translation>
-    </message>
     <message>
         <source>Send to multiple recipients at once</source>
         <translation>Gửi đến nhiều người nhận trong một lần</translation>
@@ -1183,6 +1159,9 @@ Ví của bạn chưa được mã hóa.</translation>
 <context>
     <name>UnitDisplayStatusBarControl</name>
     </context>
+<context>
+    <name>WalletController</name>
+    </context>
 <context>
     <name>WalletFrame</name>
     </context>
